"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@apollo/client/core");
const chai_1 = require("chai");
const sinon_1 = __importDefault(require("sinon"));
const SlicknodeLink_1 = __importStar(require("../SlicknodeLink"));
// tslint:disable no-unused-expression
describe('SlicknodeLink', () => {
    it('forwards request to next link', (done) => {
        const data = {
            test: true,
        };
        const nextLink = sinon_1.default.stub().callsFake((e) => {
            return new core_1.Observable((observer) => {
                observer.next({ data });
            });
        });
        const slicknodeLink = core_1.ApolloLink.from([
            new SlicknodeLink_1.default(),
            new core_1.ApolloLink(nextLink),
        ]);
        const query = core_1.gql `{test}`;
        const request = {
            query,
            variables: {},
        };
        const observable = core_1.execute(slicknodeLink, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
                chai_1.expect(nextLink.calledOnce).to.be.true;
                done();
            },
            error: done,
        });
    });
    it('forwards accessToken from options', (done) => {
        const data = {
            test: true,
        };
        const accessToken = 'abc123';
        const slicknodeLink = new SlicknodeLink_1.default({
            accessToken,
        });
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink((operation) => {
                chai_1.expect(operation.getContext()).to.deep.equal({
                    headers: {
                        Authorization: `Bearer ${accessToken}`,
                    },
                });
                done();
                return null;
            }),
        ]);
        const query = core_1.gql `{test}`;
        const request = {
            query,
            variables: {},
        };
        const observable = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
            },
            error: done,
        });
    });
    it('uses accessToken from auth token set', (done) => {
        const data = {
            test: true,
        };
        const authTokenSet = {
            accessToken: 'accessToken1',
            accessTokenLifetime: 20,
            refreshToken: 'refresh1',
            refreshTokenLifetime: 100,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        slicknodeLink.setAuthTokenSet(authTokenSet);
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink((operation) => {
                chai_1.expect(operation.getContext()).to.deep.equal({
                    headers: {
                        Authorization: `Bearer ${authTokenSet.accessToken}`,
                    },
                });
                done();
                return null;
            }),
        ]);
        const query = core_1.gql `{test}`;
        const request = {
            query,
            variables: {},
        };
        const observable = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
            },
            error: done,
        });
    });
    it('returns valid value for hasAccessToken', () => {
        const authTokenSet = {
            accessToken: 'accessToken1',
            accessTokenLifetime: 20,
            refreshToken: 'refresh1',
            refreshTokenLifetime: 100,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        chai_1.expect(slicknodeLink.hasAccessToken()).to.be.false;
        slicknodeLink.setAuthTokenSet(authTokenSet);
        chai_1.expect(slicknodeLink.hasAccessToken()).to.be.true;
    });
    it('returns valid value for hasRefreshToken', () => {
        const authTokenSet = {
            accessToken: 'accessToken1',
            accessTokenLifetime: 20,
            refreshToken: 'refresh1',
            refreshTokenLifetime: 100,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        chai_1.expect(slicknodeLink.hasRefreshToken()).to.be.false;
        slicknodeLink.setAuthTokenSet(authTokenSet);
        chai_1.expect(slicknodeLink.hasRefreshToken()).to.be.true;
    });
    it('removes access token if expire is set to NULL', () => {
        const authTokenSet = {
            accessToken: 'accessToken1',
            accessTokenLifetime: 20,
            refreshToken: 'refresh1',
            refreshTokenLifetime: 100,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        slicknodeLink.setAuthTokenSet(authTokenSet);
        chai_1.expect(slicknodeLink.hasAccessToken()).to.be.true;
        slicknodeLink.setAccessTokenExpires(null);
        chai_1.expect(slicknodeLink.hasAccessToken()).to.be.false;
    });
    it('does not use expired accessToken form auth token set', (done) => {
        const data = {
            test: true,
        };
        const authTokenSet = {
            accessToken: 'accessToken1',
            accessTokenLifetime: -20,
            refreshToken: 'refresh1',
            refreshTokenLifetime: -100,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        slicknodeLink.setAuthTokenSet(authTokenSet);
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink((operation) => {
                chai_1.expect(operation.getContext()).to.deep.equal({
                    headers: {},
                });
                done();
                return null;
            }),
        ]);
        const query = core_1.gql `{test}`;
        const request = {
            query,
            variables: {},
        };
        const observable = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
            },
            error: done,
        });
    });
    it('executes refreshToken request in subsequent links and adds auth headers', (done) => {
        const data = {
            test: true,
        };
        const authTokenSet = {
            accessToken: 'accessToken1',
            accessTokenLifetime: -20,
            refreshToken: 'refresh1',
            refreshTokenLifetime: 100,
        };
        const refreshedAuthTokenSet = {
            accessToken: 'accessToken2',
            accessTokenLifetime: 20,
            refreshToken: 'refresh2',
            refreshTokenLifetime: 200,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        slicknodeLink.setAuthTokenSet(authTokenSet);
        let refreshTokenExecuted = false;
        const query = core_1.gql `{test}`;
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink((operation) => {
                if (!refreshTokenExecuted) {
                    // First request refreshes auth tokens
                    chai_1.expect(operation.query).to.deep.equal(core_1.gql `${SlicknodeLink_1.REFRESH_TOKEN_MUTATION}`);
                    chai_1.expect(operation.variables).to.deep.equal({
                        token: authTokenSet.refreshToken,
                    });
                    chai_1.expect(operation.getContext()).to.deep.equal({});
                    refreshTokenExecuted = true;
                    return new core_1.Observable((observer) => {
                        observer.next({
                            data: {
                                refreshAuthToken: refreshedAuthTokenSet,
                            },
                        });
                    });
                }
                // Second request returns actual results and should contain auth headers
                chai_1.expect(operation.query).to.deep.equal(query);
                chai_1.expect(operation.variables).to.deep.equal({});
                chai_1.expect(operation.getContext()).to.deep.equal({
                    headers: {
                        Authorization: `Bearer ${refreshedAuthTokenSet.accessToken}`,
                    },
                });
                return new core_1.Observable((observer) => {
                    observer.next({ data });
                });
            }),
        ]);
        const request = {
            query,
            variables: {},
        };
        const observable = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
                done();
            },
            error: done,
        });
    });
    it('executes refreshToken only once for concurrent requests', (done) => {
        const data = {
            test: true,
        };
        const authTokenSet = {
            accessToken: 'accessToken1',
            accessTokenLifetime: -20,
            refreshToken: 'refresh1',
            refreshTokenLifetime: 100,
        };
        const refreshedAuthTokenSet = {
            accessToken: 'accessToken2',
            accessTokenLifetime: 20,
            refreshToken: 'refresh2',
            refreshTokenLifetime: 200,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        slicknodeLink.setAuthTokenSet(authTokenSet);
        let refreshTokenExecuted = false;
        const query = core_1.gql `{test}`;
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink((operation) => {
                if (!refreshTokenExecuted) {
                    // First request refreshes auth tokens
                    chai_1.expect(operation.query).to.deep.equal(core_1.gql `${SlicknodeLink_1.REFRESH_TOKEN_MUTATION}`);
                    chai_1.expect(operation.variables).to.deep.equal({
                        token: authTokenSet.refreshToken,
                    });
                    chai_1.expect(operation.getContext()).to.deep.equal({});
                    refreshTokenExecuted = true;
                    return new core_1.Observable((observer) => {
                        // Run in next event loop to test concurrent requests
                        setTimeout(() => {
                            observer.next({
                                data: {
                                    refreshAuthToken: refreshedAuthTokenSet,
                                },
                            });
                        }, 0);
                    });
                }
                // Second request returns actual results and should contain auth headers
                chai_1.expect(operation.query).to.deep.equal(query);
                chai_1.expect(operation.variables).to.deep.equal({});
                chai_1.expect(operation.getContext()).to.deep.equal({
                    headers: {
                        Authorization: `Bearer ${refreshedAuthTokenSet.accessToken}`,
                    },
                });
                return new core_1.Observable((observer) => {
                    observer.next({ data });
                });
            }),
        ]);
        const request = {
            query,
            variables: {},
        };
        // Execute two requests concurrently
        const observable = core_1.execute(link, request);
        const observable2 = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
                // done();
            },
            error: done,
        });
        observable2.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
                done();
            },
            error: done,
        });
    });
    it('ignores invalid refreshToken', (done) => {
        const data = {
            test: true,
        };
        const authTokenSet = {
            accessToken: 'accessToken1',
            accessTokenLifetime: -20,
            refreshToken: 'refresh1',
            refreshTokenLifetime: 100,
        };
        const refreshedAuthTokenSet = {
            accessToken: 'accessToken2',
            accessTokenLifetime: 'invalid',
            refreshToken: 'refresh2',
            refreshTokenLifetime: 200,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        slicknodeLink.setAuthTokenSet(authTokenSet);
        let refreshTokenExecuted = false;
        const query = core_1.gql `{test}`;
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink((operation) => {
                if (!refreshTokenExecuted) {
                    // First request refreshes auth tokens
                    chai_1.expect(operation.query).to.deep.equal(core_1.gql `${SlicknodeLink_1.REFRESH_TOKEN_MUTATION}`);
                    chai_1.expect(operation.variables).to.deep.equal({
                        token: authTokenSet.refreshToken,
                    });
                    chai_1.expect(operation.getContext()).to.deep.equal({});
                    refreshTokenExecuted = true;
                    return new core_1.Observable((observer) => {
                        observer.next({
                            data: {
                                refreshAuthToken: refreshedAuthTokenSet,
                            },
                        });
                    });
                }
                // Second request returns actual results and should contain auth headers
                chai_1.expect(operation.query).to.deep.equal(query);
                chai_1.expect(operation.variables).to.deep.equal({});
                chai_1.expect(operation.getContext()).to.deep.equal({ headers: {} });
                chai_1.expect(slicknodeLink.getRefreshToken()).to.be.null;
                return new core_1.Observable((observer) => {
                    observer.next({ data });
                });
            }),
        ]);
        const request = {
            query,
            variables: {},
        };
        const observable = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
                done();
            },
            error: done,
        });
    });
    it('ignores expired accessToken from automatic refresh', (done) => {
        const data = {
            test: true,
        };
        const authTokenSet = {
            accessToken: 'accessToken1',
            accessTokenLifetime: -20,
            refreshToken: 'refresh1',
            refreshTokenLifetime: 100,
        };
        const refreshedAuthTokenSet = {
            accessToken: 'accessToken2',
            accessTokenLifetime: -20,
            refreshToken: 'refresh2',
            refreshTokenLifetime: 200,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        slicknodeLink.setAuthTokenSet(authTokenSet);
        let refreshTokenExecuted = false;
        const query = core_1.gql `{test}`;
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink((operation) => {
                if (!refreshTokenExecuted) {
                    // First request refreshes auth tokens
                    chai_1.expect(operation.query).to.deep.equal(core_1.gql `${SlicknodeLink_1.REFRESH_TOKEN_MUTATION}`);
                    chai_1.expect(operation.variables).to.deep.equal({
                        token: authTokenSet.refreshToken,
                    });
                    chai_1.expect(operation.getContext()).to.deep.equal({});
                    refreshTokenExecuted = true;
                    return new core_1.Observable((observer) => {
                        observer.next({
                            data: {
                                refreshAuthToken: refreshedAuthTokenSet,
                            },
                        });
                    });
                }
                // Second request returns actual results and should contain auth headers
                chai_1.expect(operation.query).to.deep.equal(query);
                chai_1.expect(operation.variables).to.deep.equal({});
                chai_1.expect(operation.getContext()).to.deep.equal({ headers: {} });
                chai_1.expect(slicknodeLink.hasAccessToken()).to.be.false;
                return new core_1.Observable((observer) => {
                    observer.next({ data });
                });
            }),
        ]);
        const request = {
            query,
            variables: {},
        };
        const observable = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
                done();
            },
            error: done,
        });
    });
    it('adds auth token set via unnamed mutation + directive', (done) => {
        const authTokenSet = {
            accessToken: 'accessToken1',
            accessTokenLifetime: 20,
            refreshToken: 'refresh1',
            refreshTokenLifetime: 100,
        };
        const data = {
            loginMutation: authTokenSet,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink(() => {
                return new core_1.Observable((observer) => {
                    observer.next({ data });
                });
            }),
        ]);
        const query = core_1.gql `mutation {
      loginMutation @authenticate {
        accessToken
        accessTokenLifetime
        refreshToken
        refreshTokenLifetime
      }
    }`;
        const request = {
            query,
            variables: {},
        };
        const observable = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
                chai_1.expect(slicknodeLink.getAccessToken()).to.deep.equal(authTokenSet.accessToken);
                chai_1.expect(slicknodeLink.getAccessTokenExpires()).to.be.above(authTokenSet.accessTokenLifetime * 1000 + Date.now() - 1000);
                chai_1.expect(slicknodeLink.getAccessTokenExpires()).to.be.below(authTokenSet.accessTokenLifetime * 1000 + Date.now() + 1);
                chai_1.expect(slicknodeLink.getRefreshToken()).to.deep.equal(authTokenSet.refreshToken);
                chai_1.expect(slicknodeLink.getRefreshTokenExpires()).to.be.above(authTokenSet.refreshTokenLifetime * 1000 + Date.now() - 1000);
                chai_1.expect(slicknodeLink.getRefreshTokenExpires()).to.be.below(authTokenSet.refreshTokenLifetime * 1000 + Date.now() + 1);
                done();
            },
            error: done,
        });
    });
    it('adds auth token set via named mutation + directive', (done) => {
        const authTokenSet = {
            accessToken: 'accessToken1',
            accessTokenLifetime: 20,
            refreshToken: 'refresh1',
            refreshTokenLifetime: 100,
        };
        const data = {
            loginMutation: authTokenSet,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink(() => {
                return new core_1.Observable((observer) => {
                    observer.next({ data });
                });
            }),
        ]);
        const query = core_1.gql `mutation LoginMutation {
      loginMutation @authenticate {
        accessToken
        accessTokenLifetime
        refreshToken
        refreshTokenLifetime
      }
    }
    `;
        const request = {
            query,
            variables: {},
            operationName: 'LoginMutation',
        };
        const observable = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
                chai_1.expect(slicknodeLink.getAccessToken()).to.deep.equal(authTokenSet.accessToken);
                chai_1.expect(slicknodeLink.getAccessTokenExpires()).to.be.above(authTokenSet.accessTokenLifetime * 1000 + Date.now() - 1000);
                chai_1.expect(slicknodeLink.getAccessTokenExpires()).to.be.below(authTokenSet.accessTokenLifetime * 1000 + Date.now() + 1);
                chai_1.expect(slicknodeLink.getRefreshToken()).to.deep.equal(authTokenSet.refreshToken);
                chai_1.expect(slicknodeLink.getRefreshTokenExpires()).to.be.above(authTokenSet.refreshTokenLifetime * 1000 + Date.now() - 1000);
                chai_1.expect(slicknodeLink.getRefreshTokenExpires()).to.be.below(authTokenSet.refreshTokenLifetime * 1000 + Date.now() + 1);
                done();
            },
            error: done,
        });
    });
    it('ignores invalid result from mutation field via authentication directive', (done) => {
        const data = {
            loginMutation: {},
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink(() => {
                return new core_1.Observable((observer) => {
                    observer.next({ data });
                });
            }),
        ]);
        const query = core_1.gql `mutation {
      loginMutation @authenticate {
        accessToken
        accessTokenLifetime
        refreshToken
        refreshTokenLifetime
      }
    }`;
        const request = {
            query,
            variables: {},
        };
        const observable = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
                chai_1.expect(slicknodeLink.getAccessToken()).to.be.null;
                chai_1.expect(slicknodeLink.getAccessTokenExpires()).to.be.null;
                chai_1.expect(slicknodeLink.getRefreshToken()).to.be.null;
                chai_1.expect(slicknodeLink.getRefreshTokenExpires()).to.be.null;
                done();
            },
            error: done,
        });
    });
    it('ignores NULL result from mutation field via authentication directive', (done) => {
        const data = {
            loginMutation: null,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink(() => {
                return new core_1.Observable((observer) => {
                    observer.next({ data });
                });
            }),
        ]);
        const query = core_1.gql `mutation {
      loginMutation @authenticate {
        accessToken
        accessTokenLifetime
        refreshToken
        refreshTokenLifetime
      }
    }`;
        const request = {
            query,
            variables: {},
        };
        const observable = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
                chai_1.expect(slicknodeLink.getAccessToken()).to.be.null;
                chai_1.expect(slicknodeLink.getAccessTokenExpires()).to.be.null;
                chai_1.expect(slicknodeLink.getRefreshToken()).to.be.null;
                chai_1.expect(slicknodeLink.getRefreshTokenExpires()).to.be.null;
                done();
            },
            error: done,
        });
    });
    it('ignores invalid auth token type result from mutation field via authentication directive', (done) => {
        const data = {
            loginMutation: {
                accessToken: 'accessToken1',
                accessTokenLifetime: '20',
                refreshToken: 'refresh1',
                refreshTokenLifetime: 100,
            },
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink(() => {
                return new core_1.Observable((observer) => {
                    observer.next({ data });
                });
            }),
        ]);
        const query = core_1.gql `mutation {
      loginMutation @authenticate {
        accessToken
        accessTokenLifetime
        refreshToken
        refreshTokenLifetime
      }
    }`;
        const request = {
            query,
            variables: {},
        };
        const observable = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
                chai_1.expect(slicknodeLink.getAccessToken()).to.be.null;
                chai_1.expect(slicknodeLink.getAccessTokenExpires()).to.be.null;
                chai_1.expect(slicknodeLink.getRefreshToken()).to.be.null;
                chai_1.expect(slicknodeLink.getRefreshTokenExpires()).to.be.null;
                done();
            },
            error: done,
        });
    });
    it('ignores authentication directive in query', (done) => {
        const authTokenSet = {
            accessToken: 'accessToken1',
            accessTokenLifetime: 20,
            refreshToken: 'refresh1',
            refreshTokenLifetime: 100,
        };
        const data = {
            loginMutation: authTokenSet,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink(() => {
                return new core_1.Observable((observer) => {
                    observer.next({ data });
                });
            }),
        ]);
        const query = core_1.gql `query LoginMutation {
      loginMutation @authenticate {
        accessToken
        accessTokenLifetime
        refreshToken
        refreshTokenLifetime
      }
    }
    `;
        const request = {
            query,
            variables: {},
            operationName: 'LoginMutation',
        };
        const observable = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
                chai_1.expect(slicknodeLink.getAccessToken()).to.be.null;
                chai_1.expect(slicknodeLink.getAccessTokenExpires()).to.be.null;
                chai_1.expect(slicknodeLink.getRefreshToken()).to.be.null;
                chai_1.expect(slicknodeLink.getRefreshTokenExpires()).to.be.null;
                done();
            },
            error: done,
        });
    });
    it('ignores login mutation without authenticate directive', (done) => {
        const authTokenSet = {
            accessToken: 'accessToken1',
            accessTokenLifetime: 20,
            refreshToken: 'refresh1',
            refreshTokenLifetime: 100,
        };
        const data = {
            loginMutation: authTokenSet,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink(() => {
                return new core_1.Observable((observer) => {
                    observer.next({ data });
                });
            }),
        ]);
        const query = core_1.gql `mutation LoginMutation {
      loginMutation {
        accessToken
        accessTokenLifetime
        refreshToken
        refreshTokenLifetime
      }
    }
    `;
        const request = {
            query,
            variables: {},
            operationName: 'LoginMutation',
        };
        const observable = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
                chai_1.expect(slicknodeLink.getAccessToken()).to.be.null;
                chai_1.expect(slicknodeLink.getAccessTokenExpires()).to.be.null;
                chai_1.expect(slicknodeLink.getRefreshToken()).to.be.null;
                chai_1.expect(slicknodeLink.getRefreshTokenExpires()).to.be.null;
                done();
            },
            error: done,
        });
    });
    it('adds auth token set via directive on alias field', (done) => {
        const authTokenSet = {
            accessToken: 'accessToken1',
            accessTokenLifetime: 20,
            refreshToken: 'refresh1',
            refreshTokenLifetime: 100,
        };
        const data = {
            alias: authTokenSet,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        const dataLoaderStub = sinon_1.default.stub();
        const nextLink = sinon_1.default.stub().callsFake(() => {
            return new core_1.Observable((observer) => {
                dataLoaderStub();
                observer.next({ data });
            });
        });
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink(nextLink),
        ]);
        const query = core_1.gql `mutation LoginMutation {
      alias: loginMutation @authenticate {
        accessToken
        accessTokenLifetime
        refreshToken
        refreshTokenLifetime
      }
    }
    `;
        const request = {
            query,
            variables: {},
            operationName: 'LoginMutation',
        };
        const observable = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
                chai_1.expect(slicknodeLink.getAccessToken()).to.deep.equal(authTokenSet.accessToken);
                chai_1.expect(slicknodeLink.getAccessTokenExpires()).to.be.above(authTokenSet.accessTokenLifetime * 1000 + Date.now() - 1000);
                chai_1.expect(slicknodeLink.getAccessTokenExpires()).to.be.below(authTokenSet.accessTokenLifetime * 1000 + Date.now() + 1);
                chai_1.expect(slicknodeLink.getRefreshToken()).to.deep.equal(authTokenSet.refreshToken);
                chai_1.expect(slicknodeLink.getRefreshTokenExpires()).to.be.above(authTokenSet.refreshTokenLifetime * 1000 + Date.now() - 1000);
                chai_1.expect(slicknodeLink.getRefreshTokenExpires()).to.be.below(authTokenSet.refreshTokenLifetime * 1000 + Date.now() + 1);
                chai_1.expect(nextLink.calledOnce).to.be.true;
                chai_1.expect(dataLoaderStub.calledOnce).to.be.true;
                done();
            },
            error: done,
        });
    });
    it('adds auth token set for multiple mutations in one query', (done) => {
        const authTokenSet = {
            accessToken: 'accessToken1',
            accessTokenLifetime: 20,
            refreshToken: 'refresh1',
            refreshTokenLifetime: 100,
        };
        const data = {
            otherMutation: true,
            loginMutation: authTokenSet,
            test: true,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink(() => {
                return new core_1.Observable((observer) => {
                    observer.next({ data });
                });
            }),
        ]);
        const query = core_1.gql `mutation LoginMutation {
      otherMutation
      loginMutation @authenticate {
        accessToken
        accessTokenLifetime
        refreshToken
        refreshTokenLifetime
      }
      test: otherMutation2
    }
    `;
        const request = {
            query,
            variables: {},
            operationName: 'LoginMutation',
        };
        const observable = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
                chai_1.expect(slicknodeLink.getAccessToken()).to.deep.equal(authTokenSet.accessToken);
                chai_1.expect(slicknodeLink.getAccessTokenExpires()).to.be.above(authTokenSet.accessTokenLifetime * 1000 + Date.now() - 1000);
                chai_1.expect(slicknodeLink.getAccessTokenExpires()).to.be.below(authTokenSet.accessTokenLifetime * 1000 + Date.now() + 1);
                chai_1.expect(slicknodeLink.getRefreshToken()).to.deep.equal(authTokenSet.refreshToken);
                chai_1.expect(slicknodeLink.getRefreshTokenExpires()).to.be.above(authTokenSet.refreshTokenLifetime * 1000 + Date.now() - 1000);
                chai_1.expect(slicknodeLink.getRefreshTokenExpires()).to.be.below(authTokenSet.refreshTokenLifetime * 1000 + Date.now() + 1);
                done();
            },
            error: done,
        });
    });
    it('removes auth tokens from link on logout', (done) => {
        const data = {
            logoutUser: {
                success: true,
            },
        };
        const authTokenSet = {
            accessToken: 'accessToken1',
            accessTokenLifetime: 20,
            refreshToken: 'refresh1',
            refreshTokenLifetime: 100,
        };
        const slicknodeLink = new SlicknodeLink_1.default();
        slicknodeLink.setAuthTokenSet(authTokenSet);
        const link = core_1.ApolloLink.from([
            slicknodeLink,
            new core_1.ApolloLink(() => {
                return new core_1.Observable((observer) => {
                    observer.next({ data });
                });
            }),
        ]);
        const query = core_1.gql `mutation LogoutMutation($token: String) {
      logoutUser(input: {refreshToken: $token}) {
        success
      }
    }`;
        const request = {
            query,
            variables: {
                token: authTokenSet.refreshToken,
            },
        };
        const observable = core_1.execute(link, request);
        observable.subscribe({
            next(result) {
                chai_1.expect(result.data).to.equal(data);
                chai_1.expect(slicknodeLink.getAccessToken()).to.be.null;
                chai_1.expect(slicknodeLink.getAccessTokenExpires()).to.be.null;
                chai_1.expect(slicknodeLink.getRefreshToken()).to.be.null;
                chai_1.expect(slicknodeLink.getRefreshTokenExpires()).to.be.null;
                done();
            },
            error: done,
        });
    });
    it('throws error if SlicknodeLink is last link in chain', () => {
        const slicknodeLink = new SlicknodeLink_1.default();
        const link = core_1.ApolloLink.from([
            slicknodeLink,
        ]);
        const query = core_1.gql `{test}`;
        const request = {
            query,
            variables: {},
        };
        chai_1.expect(() => {
            core_1.execute(link, request);
        }).to.throw('Network link is missing in apollo client or SlicknodeLink is last link in the chain.');
    });
});
//# sourceMappingURL=index-test.js.map